#!/usr/bin/perl

# Script to ensure that the PopcornHour A-100 (and possibly all other Syabas-based
# devices) discovers this SwissCenter server. It works by detecting the UPnP SERVER 
# response for A-100 and sending a URL request.
#
# This script is based on the original broadcast.pl included with SwissCenter
# (http://swisscenter.co.uk/).
#
# Changes by Keith Solomon - 09/11/2008
#
# If you receive an error that "IO::Socket::Multicast" doesn't exist, then install it
# as follow (as the root user):
#   perl -MCPAN -e 'install IO::Interface'
#   perl -MCPAN -e 'install IO::Socket::Multicast'
#   perl -MCPAN -e 'install HTTP::Lite'

# Bring in a couple of modules
use IO::Socket::Multicast;
use HTTP::Lite;
use Getopt::Long;

### BEGING SCRIPT CONFIG ###
# Change this to reflect your SC server's IP and display name
# host - Name shown on Source screen
# addr - address (name or IP, IP tends to work better) and port (defaut 8080)

my $host = "SwissCenter Linux Media Server";
my $addr = "swissaddr:port";

### END SCRIPT CONFIG ###

$result = GetOptions("host=s" => \$host, "ip=s" => \$addr, "debug" => \$debug);

# the upnp multicast group:port
use constant GROUP => '239.255.255.250';
use constant PORT  => '1900';

# and create the multicast endpoint
print "G=".GROUP." P=".PORT." \n" if ($debug);

my $sock = IO::Socket::Multicast->new(Proto=>'udp',LocalPort=>PORT);
$sock->mcast_add(GROUP) || die "Couldn't set multicast UPnP Group: $!\n";

# should really listen for a command to stop
while (1) {
    my $data;
    # read a multicast message
    $hispaddr = $sock->recv($data,1024);
    ($port, $hisiaddr) = sockaddr_in($hispaddr);
    $peer_addr = inet_ntoa($hisiaddr);
 
    #print "Recv from ".$peer_addr.":".$hisaddr.":".$port.":".$data;

    if ($data =~ /NOTIFY/) {
        # Found a PopcornHour (or possibly another Syabas-based) Device
        if ($data =~ /SERVER: UCosII, UPnP\/1.0, Syabas myiBox UPnP-stack, v1.0/) {
            # Find out where it is
            $data =~ /LOCATION: (.*)/;
            $url=$1;
            sleep 1;
            $http = new HTTP::Lite;
            $req = $http->request($url."?POSTSyabasiBoxURLpeername=$host&peeraddr=$addr&")
              or die "Unable to get document: $!";
            print $http->body() if ($debug);
        }
    }
}
